/*
 * MapFrame.java
 *
 * Created on 25. �erven 2007, 17:07
 */

package gui;

import pathfinder.JMapPanel;
import pathfinder.JPFMap;
import pathfinder.JPathFinder;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.io.File;
import java.util.Timer;
import java.util.TimerTask;
import javax.swing.JOptionPane;

/**
 *
 * @author  Petr Buryan
 */
public class MapFrame extends javax.swing.JFrame {
    JMapPanel m_mapPanel = null;
    JPFMap m_map=null;
    JPathFinder m_pathFinder;
    int m_mapPanelID=-1;
    Thread m_searchThread;

    private static final int PF_DFS=0;
    private static final int PF_BFS=1;
    private static final int PF_ASP=2;
    private static final int PF_AST=3;

    /** Creates new form MapFrame
     * @param pathFinder the class containig the search algorithms
     */
    public MapFrame(JPathFinder pathFinder) {
        initComponents();
        m_pathFinder=pathFinder;
        m_pathFinder.setInformationArea(infoTextArea);
    }

    /** Creates new form MapFrame
     *
     * @param pathFinder the class containig the search algorithms
     * @param fName file with the map
     */
    public MapFrame(JPathFinder pathFinder, String fName) {
        initComponents();
        m_pathFinder=pathFinder;
        infoTextArea.setText("");
        m_pathFinder.setInformationArea(infoTextArea);
        File mapFile=new File(fName);
        if(!mapFile.exists()){
            JOptionPane.showMessageDialog(this, "Error loading map! "+mapFile+" does not exist!", "Error loading map", JOptionPane.ERROR_MESSAGE);
            return;
        }
        m_map=new JPFMap(mapFile);
        mapRefreshButton.setEnabled(true);
        unblockButton.setEnabled(true);
        drawMap();
        inform("Map loaded from "+mapFile.getAbsolutePath());
    }


    /**
     * Redraws the map
     */
    public void drawMap(){
        Container content_pane = getContentPane ();
        
        if (m_mapPanelID>0){
                content_pane.remove(m_mapPanelID);
                m_mapPanelID=-1;
        }
         Dimension size=this.getSize();
         double h=size.getHeight();
         double w=size.getWidth();
         int width=Integer.valueOf((int)Math.round(w-50.0));
         int height=Integer.valueOf((int)Math.round(h-200.0));
         m_mapPanel = m_map.drawMap(width,height);
         m_mapPanel.setBounds(15,15,width,height);
         m_mapPanel.setVisible(true);
         m_mapPanel.setParentFrame(this);
         m_mapPanelID=content_pane.getComponentCount();
         content_pane.add(m_mapPanel,m_mapPanelID);
        m_mapPanel.paintComponent(m_mapPanel.getGraphics());
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        exitButton = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        bfsButton = new javax.swing.JButton();
        dfsButton = new javax.swing.JButton();
        startTextField = new javax.swing.JTextField();
        stopTextField = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        jSeparator3 = new javax.swing.JSeparator();
        shortestPathButton = new javax.swing.JButton();
        shortestTimeButton = new javax.swing.JButton();
        jSeparator4 = new javax.swing.JSeparator();
        stopButton = new javax.swing.JButton();
        jButton1 = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        mapRefreshButton = new javax.swing.JButton();
        unblockButton = new javax.swing.JButton();
        jSeparator2 = new javax.swing.JSeparator();
        mapLoadButton = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        infoTextArea = new javax.swing.JTextArea();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("PathFinder");
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                formMouseClicked(evt);
            }
        });
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        });

        exitButton.setText("Exit");
        exitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitButtonActionPerformed(evt);
            }
        });

        jPanel1.setBorder(javax.swing.BorderFactory.createEtchedBorder(new java.awt.Color(255, 255, 255), new java.awt.Color(204, 204, 204)));

        bfsButton.setText("BFS");
        bfsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bfsButtonActionPerformed(evt);
            }
        });

        dfsButton.setText("DFS");
        dfsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dfsButtonActionPerformed(evt);
            }
        });

        startTextField.setText("61");
        startTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startTextFieldActionPerformed(evt);
            }
        });

        stopTextField.setText("13");

        jLabel1.setText("Starting city:");

        jLabel2.setText("Target city:");

        jSeparator1.setOrientation(javax.swing.SwingConstants.VERTICAL);

        jSeparator3.setOrientation(javax.swing.SwingConstants.VERTICAL);

        shortestPathButton.setText("Shortest Path");
        shortestPathButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                shortestPathButtonActionPerformed(evt);
            }
        });

        shortestTimeButton.setText("Shortest Time");
        shortestTimeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                shortestTimeButtonActionPerformed(evt);
            }
        });

        jSeparator4.setOrientation(javax.swing.SwingConstants.VERTICAL);

        stopButton.setText("Stop Search");
        stopButton.setEnabled(false);
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed(evt);
            }
        });

        jButton1.setText("...");
        jButton1.setEnabled(false);
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BaseSearchActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 63, Short.MAX_VALUE)
                    .add(jLabel2))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(stopTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 26, Short.MAX_VALUE)
                    .add(startTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 26, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSeparator1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 7, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(bfsButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 52, Short.MAX_VALUE)
                    .add(dfsButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 52, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSeparator3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 11, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(shortestPathButton)
                    .add(shortestTimeButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 100, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSeparator4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 11, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(jButton1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(stopButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 116, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jSeparator3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 82, Short.MAX_VALUE)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(stopButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 14, Short.MAX_VALUE)
                .add(jButton1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
            .add(jSeparator4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 82, Short.MAX_VALUE)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jSeparator1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 60, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel1Layout.createSequentialGroup()
                        .add(shortestPathButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 14, Short.MAX_VALUE)
                        .add(shortestTimeButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel1Layout.createSequentialGroup()
                        .add(bfsButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 14, Short.MAX_VALUE)
                        .add(dfsButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel1Layout.createSequentialGroup()
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(jPanel1Layout.createSequentialGroup()
                                .add(6, 6, 6)
                                .add(jLabel1))
                            .add(startTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(20, 20, 20)
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE, false)
                            .add(jLabel2)
                            .add(stopTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap())
        );

        jPanel1Layout.linkSize(new java.awt.Component[] {jButton1, stopButton}, org.jdesktop.layout.GroupLayout.VERTICAL);

        jPanel2.setBorder(javax.swing.BorderFactory.createEtchedBorder(new java.awt.Color(255, 255, 255), new java.awt.Color(204, 204, 204)));

        mapRefreshButton.setText("Refresh map");
        mapRefreshButton.setEnabled(false);
        mapRefreshButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mapRefreshButtonActionPerformed(evt);
            }
        });

        unblockButton.setText("Unblock Roads");
        unblockButton.setEnabled(false);
        unblockButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                unblockButtonActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, unblockButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 111, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, mapRefreshButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 111, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(mapRefreshButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(unblockButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(14, Short.MAX_VALUE))
        );

        mapLoadButton.setText("Load & Draw Map");
        mapLoadButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mapLoadButtonActionPerformed(evt);
            }
        });

        jScrollPane1.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        infoTextArea.setColumns(20);
        jScrollPane1.setViewportView(infoTextArea);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 192, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(exitButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 80, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(mapLoadButton)))
                    .add(jSeparator2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 912, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap(412, Short.MAX_VALUE)
                .add(jSeparator2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                        .add(layout.createSequentialGroup()
                            .add(mapLoadButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 54, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                            .add(exitButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 84, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
        setBackground(getBackground());
        if(m_map!=null)
            drawMap();
    }//GEN-LAST:event_formComponentResized



    private void shortestTimeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_shortestTimeButtonActionPerformed
        infoTextArea.setText("Performing A* for time: \n");
        runSearch(PF_AST);
    }//GEN-LAST:event_shortestTimeButtonActionPerformed

    private void shortestPathButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_shortestPathButtonActionPerformed
        infoTextArea.setText("Performing A* for path: \n");
        runSearch(PF_ASP);
    }//GEN-LAST:event_shortestPathButtonActionPerformed

    private void startTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startTextFieldActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_startTextFieldActionPerformed

    private void mapRefreshButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapRefreshButtonActionPerformed
        if(m_map!=null){
            drawMap();}
        
    }//GEN-LAST:event_mapRefreshButtonActionPerformed

    private void dfsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_dfsButtonActionPerformed
        infoTextArea.setText("Performing DFS:\n");
        runSearch(PF_DFS);
    }//GEN-LAST:event_dfsButtonActionPerformed

    private void bfsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bfsButtonActionPerformed
        infoTextArea.setText("Performing BFS: \n");
        runSearch(PF_BFS);
    }//GEN-LAST:event_bfsButtonActionPerformed

    private void mapLoadButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapLoadButtonActionPerformed
        m_map=new JPFMap();
        mapRefreshButton.setEnabled(true);
        unblockButton.setEnabled(true);
        drawMap();
        infoTextArea.setText("");
        inform("Map loaded..");
    }//GEN-LAST:event_mapLoadButtonActionPerformed

    private void exitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitButtonActionPerformed
        System.exit(0);
    }//GEN-LAST:event_exitButtonActionPerformed

    private void BaseSearchActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_BaseSearchActionPerformed
        //TODO
}//GEN-LAST:event_BaseSearchActionPerformed

    private void formMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseClicked
       
    }

    /**
     *  internal purposes
     * @param searchThread
     */
    public void setSearchThread(Thread searchThread) {
        this.m_searchThread = searchThread;
        stopButton.setEnabled(true);
    }

    /**
     *  Runs the search of a given type in separate thread
     * @param type
     */
    public void runSearch(final int type){
        final int start=Integer.valueOf(startTextField.getText());
        final int stop=Integer.valueOf(stopTextField.getText());
        Thread t=new Thread("Path Finder"){
            @Override
            public void run(){
                switch(type){
                    case PF_DFS:
                        m_pathFinder.performDFS(start,stop,m_map);
                        break;
                    case PF_BFS:
                        m_pathFinder.performBFS(start,stop,m_map);
                        break;
                    case PF_ASP:
                        m_pathFinder.searchForShortestPath(start,stop,m_map);
                        break;
                    case PF_AST:
                        m_pathFinder.searchForShortestTime(start,stop,m_map);
                        break;
                }
                
                stopButton.setEnabled(false);
            };
        };setSearchThread(t);
        try{
            t.start();
        } catch(Exception e){
            JOptionPane.showMessageDialog(this, "An error occured: "+e, "Error!", JOptionPane.ERROR_MESSAGE);
            inform("An error occured: ");
            inform(e.toString());
            stopButton.setEnabled(false);
        }
    }

    /** A method for city selection with mouse.
     * Blinks in the proper textfield
     * @param cityId
     */
    public void setStopCity(int cityId) {
        final Color col = stopTextField.getBackground();
        this.stopTextField.setBackground(Color.red);
        this.stopTextField.setText(cityId+"");
        Timer t=new Timer();
            t.schedule(new TimerTask() {
                @Override
                public void run() {
                    stopTextField.setBackground(col);
                }
            }, 500);
    }

    /** A method for city selection with mouse.
     * Blinks in the proper textfield
     * @param cityId
     */
    public void setStartCity(int cityId) {
        
            final Color col = startTextField.getBackground();
            this.startTextField.setBackground(Color.green.darker());
            this.startTextField.setText(cityId + "");
            Timer t=new Timer();
            t.schedule(new TimerTask() {
                @Override
                public void run() {
                    startTextField.setBackground(col);
                }
            }, 500);
    }//GEN-LAST:event_formMouseClicked

    private void unblockButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_unblockButtonActionPerformed
        if(m_map!=null){
            m_map.unblockRoads();
            drawMap();}
    }//GEN-LAST:event_unblockButtonActionPerformed

    private void stopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed
        if(m_searchThread.isAlive())
            m_searchThread.stop();
        stopButton.setEnabled(false);
    }//GEN-LAST:event_stopButtonActionPerformed

    /** Writes the message to the information window of the GUI
     *
     * @param message
     */
    public void inform(String message){
        if(infoTextArea==null)
            return;
        infoTextArea.append(message);
        infoTextArea.append("\n");
        infoTextArea.setCaretPosition(infoTextArea.getText().length());
    }

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bfsButton;
    private javax.swing.JButton dfsButton;
    private javax.swing.JButton exitButton;
    private javax.swing.JTextArea infoTextArea;
    private javax.swing.JButton jButton1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JButton mapLoadButton;
    private javax.swing.JButton mapRefreshButton;
    private javax.swing.JButton shortestPathButton;
    private javax.swing.JButton shortestTimeButton;
    private javax.swing.JTextField startTextField;
    private javax.swing.JButton stopButton;
    private javax.swing.JTextField stopTextField;
    private javax.swing.JButton unblockButton;
    // End of variables declaration//GEN-END:variables
    
}
